--[[

   menu.lua
   Main menu of Because it's fun, Fay.
	
   Copyright 2008 Oxeye Game Studio. http://www.oxeyegames.com
	
   File contributors:
      Jens Bergensten

   This game was written for the TIGSource Procedural Generation
   competition in May 2008. You may use this Lua source code in
   any way you please, we don't mind. However, the game's sound
   effects and artwork belong to us, so please ask for permission
   before you do anything funny =)

]]

local because = {
	sprite = nil,
	x = 512,
	y = 300,
	rotation = 0,
	scale = 0,
	alpha = 0
}
local fay = {
	sprite = nil,
	x = 512,
	y = 400,
	rotation = 0,
	scale = 0,
	alpha = 0
}
local creditPos = {}
local introTimer = 0
local showTexts = false
local circle
local fadeOut = false
local fadeOutTimer = 0

local function menuStartSinglePlayer()
	singlePlayer = true
	fadeOut = true
end
local function menuStartMultiPlayer()
	singlePlayer = false
	fadeOut = true
end
local function menuExitGame()
	daisy.exitGame()
end

local menuItems = {
	{ text = "One Player", action = menuStartSinglePlayer, info = "Controls: Arrow Keys and Space" },
	{ text = "Two Players (Coop)", action = menuStartMultiPlayer, info = "Controls: WASD and Arrow Keys" },
	{ text = "Exit", action = menuExitGame, info = "You WOULDN'T! :'(" }
}
local menuIndex = 1
local nav = {
	up = false, prevUp = false,
	down = false, prevDown = false,
	enter = false, prevEnter = false
}

local clouds = {}

local function nextState()
	--daisy.stopAllMusic()
	unregisterMenuState()
	registerGameState()
end


local function menuRun(time)

	introTimer = introTimer + time

	if introTimer < 4.0 then
		because.scale = introTimer / 4
		because.alpha = (introTimer / 4) * 255
		because.rotation = math.pi * 8 * introTimer

		local fayTimer = introTimer - 1
		if fayTimer > 0 then
			fay.scale = fayTimer / 3
			fay.alpha = (fayTimer / 3) * 255
			fay.rotation = math.pi * 2 * fayTimer
		end
	else
		because.scale, because.rotation, because.alpha = 1.0, 0, 255
		fay.scale, fay.rotation, fay.alpha = 1.0, 0, 255
	end

	because.x = 500 + math.cos(introTimer * .4) * 25
	because.y = 160 + math.sin(introTimer * .2) * 20
	fay.x = 528 + math.sin(introTimer * .2) * 25
	fay.y = 240 + math.cos(introTimer * .1) * 10

	for index,cloud in pairs(clouds) do
		for i=1,#cloud do
			cloud[i].x = cloud[i].x + cloud[i].vx * time
			cloud[i].y = cloud[i].y + cloud[i].vy * time
		end
	end

	if fadeOut then
		fadeOutTimer = fadeOutTimer + time
		daisy.updateMusic("29.mp3", 1.0 - (fadeOutTimer / 4))
		if fadeOutTimer >= 2 then
			nextState()
		end
	end

	if introTimer > 6 then
		showTexts = true
	end

	nav.up, nav.down, nav.enter = daisy.isKeyPressed(38), daisy.isKeyPressed(40), daisy.isKeyPressed(13) or daisy.isKeyPressed(32)
	if nav.enter and not nav.prevEnter then
		if not showTexts then
			showTexts = true
		elseif menuItems[menuIndex].action then
			menuItems[menuIndex].action()
		end
	end
	if nav.up and not nav.prevUp then
		menuIndex = menuIndex - 1
		if menuIndex <= 0 then
			menuIndex = #menuItems
		end
	end
	if nav.down and not nav.prevDown then
		menuIndex = menuIndex + 1
		if menuIndex > #menuItems then
			menuIndex = 1
		end
	end
	nav.prevUp, nav.prevDown, nav.prevEnter = nav.up, nav.down, nav.enter

end

local function menuRender()

	daisy.renderRectangle(0, 0, 1024, 768, 255, 130, 180, 230)

	local topCount = ((introTimer-1) / 3) * 100
	for index,cloud in pairs(clouds) do
		for i=1,#cloud do
			if i >= topCount then
				break
			end
			daisy.renderSpriteState(circle, cloud[i].x, cloud[i].y, cloud[i].size, 0, 96, 255, 255, 255)
		end
	end

	daisy.renderSpriteState(because.sprite, because.x, because.y, because.scale, because.rotation, because.alpha)
	daisy.renderSpriteState(fay.sprite, fay.x, fay.y, fay.scale, fay.rotation, fay.alpha)

	if showTexts then

		local y = 400
		for i=1, #menuItems do
			if i == menuIndex then
				daisy.renderRectangle(412, y - 2, 200, 18, 128, 0, 0, 0)
			end
			daisy.renderText(menuItems[i].text, 512, y, 1, "gameFont.fnt", 255, 255, 255, 160)
			y = y + 20
		end
		y = y + 20
		daisy.renderText(menuItems[menuIndex].info, 512, y, 1, "gameFont.fnt", 255, 160, 160, 100)

		daisy.renderText("Game Design", -128 + creditPos[1] * 256, 700, 1, "gameFont.fnt", 255, 255, 255, 160)
		daisy.renderText("Oxeye Game Studio", -128 + creditPos[1] * 256, 715, 1, "gameFont.fnt", 255, 255, 255, 160)

		daisy.renderText("Programming", -128 + creditPos[2] * 256, 700, 1, "gameFont.fnt", 255, 255, 255, 160)
		daisy.renderText("Jens Bergensten", -128 + creditPos[2] * 256, 715, 1, "gameFont.fnt", 255, 255, 255, 160)

		daisy.renderText("Artwork", -128 + creditPos[3] * 256, 700, 1, "gameFont.fnt", 255, 255, 255, 160)
		daisy.renderText("Daniel Brynolf", -128 + creditPos[3] * 256, 715, 1, "gameFont.fnt", 255, 255, 255, 160)
		daisy.renderText("Pontus Hammarberg", -128 + creditPos[3] * 256, 730, 1, "gameFont.fnt", 255, 255, 255, 160)
		daisy.renderText("Nils Carlen", -128 + creditPos[3] * 256, 745, 1, "gameFont.fnt", 255, 255, 255, 160)

		daisy.renderText("Music", -128 + creditPos[4] * 256, 700, 1, "gameFont.fnt", 255, 255, 255, 160)
		--daisy.renderText("To Be Determined", -128 + creditPos[4] * 256, 715, 1, "gameFont.fnt", 255, 255, 255, 160)
		daisy.renderText("Izuho Takeuchi", -128 + creditPos[4] * 256, 715, 1, "gameFont.fnt", 255, 255, 255, 160)
		daisy.renderText("(Phantasy Star 3)", -128 + creditPos[4] * 256, 730, 1, "gameFont.fnt", 255, 255, 255, 160)
	end

	if fadeOut then
		local alpha = (fadeOutTimer / 2) * 255
		daisy.renderRectangle(0, 0, 1024, 768, alpha, 0, 0, 0)
	end
end


local function generateCloud(cloud, startX, startY)

	local cloudSpeed = -.5 - math.random() * 2
	cloud[1] = { x=startX,y=startY,size=1,vx=-1 - math.random() * 3, vy=cloudSpeed }
	local count = math.random(50,100)
	for i=2,count do

		local selection = math.random(#cloud)
		local x = cloud[selection].x + math.cos(math.random() * math.pi * 2) * 16 * cloud[selection].size
		local y = cloud[selection].y + math.cos(math.random() * math.pi * 2) * 14 * cloud[selection].size
		local size = 0.5 + math.random()

		cloud[i] = { x=x, y=y, size=size, vx=-1 - math.random() * 3, vy=cloudSpeed }
	end

	-- fast-forward time a little
	for i=1,count do
		cloud[i].x = cloud[i].x + cloud[i].vx * 30
		cloud[i].y = cloud[i].y + cloud[i].vy * 30
	end
end

function registerMenuState()
	hook.add("frameUpdate", menuRun)
	hook.add("frameRender", menuRender)

	fay.sprite = daisy.createSpriteState("Fay", "menuArt.dat")
	because.sprite = daisy.createSpriteState("Because", "menuArt.dat")
	circle = daisy.createSpriteState("Circle", "itt.dat")

	creditPos[1] = math.random(1,4)
	for i=2,4 do
		local ok = true
		repeat
			ok = true
			creditPos[i] = math.random(1,4)
			for j=1,(i-1) do
				if creditPos[i] == creditPos[j] then
					ok = false
					break
				end
			end
		until ok
	end

	for i=1,25 do
		clouds[i] = {}
		generateCloud(clouds[i], 100 + math.random(1, 1300), 100 + math.random(1, 900))
	end

	daisy.playMusic("29.mp3")
end

function unregisterMenuState()
	hook.remove("frameUpdate", menuRun)
	hook.remove("frameRender", menuRender)
end
